﻿namespace TabsStudioMarker
{
    class SolutionPath
    {
        public SolutionPath(EnvDTE80.DTE2 dte_)
        {
            dte = dte_;
        }

        public string RelativePath(WindowDocument wd)
        {
            string absolutePath = wd.document;
            try
            {
                if (!System.IO.Path.IsPathRooted(absolutePath))
                    return absolutePath;
            }
            catch (System.ArgumentException) // invalid characters 
            {
                return absolutePath;
            }

            if (!IsSolutionReady())
                return absolutePath;

            string result = FileUtils.GetLocalPath(GetSolutionDirectory(), absolutePath);
            if (result == null && wd.projectID != null && wd.projectPath != null)
            {
                string projectPath = FileUtils.GetRelativePath(wd.projectPath, absolutePath);
                if (projectPath != null)
                    result = "*" + wd.projectID + "*" + projectPath;
            }
            if (result == null && wd.solutionItem)
                result = FileUtils.GetRelativePath(GetSolutionDirectory(), absolutePath);
            if (result == null)
                return absolutePath;

            return result;
        }

        private bool IsSolutionReady()
        {
            return IsSolutionOpen() && IsSolutionPathValid();
        }

        private bool IsSolutionOpen()
        {
            return dte != null && dte.Solution != null && dte.Solution.IsOpen;
        }

        private bool IsSolutionPathValid()
        {
            try
            {
                string path = dte.Solution.FileName;
                return path != null && path.Length > 0 && System.IO.Path.IsPathRooted(path) && System.IO.File.Exists(path);
            }
            catch (System.Exception)
            {
            }
            return false;
        }

        private string GetSolutionDirectory()
        {
            return System.IO.Path.GetDirectoryName(GetSolutionFile());
        }

        private string GetSolutionFile()
        {
            return dte.Solution.FileName;
        }

        private EnvDTE80.DTE2 dte;
    }
}
