﻿namespace TabsStudioMarker
{
    using HighlightedTabList = System.Collections.Generic.List<TabHighlightState>;

    class SolutionStateSaver
    {
        public SolutionStateSaver(EnvDTE80.DTE2 dte_, Model model_)
        {
            model = model_;
            dte = dte_;
        }

        public void Save()
        {
            HighlightedTabList tabList = model.GetTabs();
            try
            {
                if (tabList.Count == 0)
                {
                    if (System.IO.File.Exists(GetStoragePath()))
                        System.IO.File.Delete(GetStoragePath());
                }
                else
                {
                    using (System.IO.FileStream stream =
                        new System.IO.FileStream(GetStoragePath(), System.IO.FileMode.Create))
                    {
                        System.Xml.Serialization.XmlSerializer serializer =
                            new System.Xml.Serialization.XmlSerializer(typeof(HighlightedTabList));
                        serializer.Serialize(stream, tabList);
                    }
                }
            }
            catch (System.Exception)
            {
            }
        }

        public void Load()
        {
            HighlightedTabList tabList = null;
            try
            {
                if (!System.IO.File.Exists(GetStoragePath()))
                    return;

                using (System.IO.FileStream stream =
                    new System.IO.FileStream(GetStoragePath(), System.IO.FileMode.Open,
                        System.IO.FileAccess.Read, System.IO.FileShare.Read))
                {
                    System.Xml.Serialization.XmlSerializer serializer =
                        new System.Xml.Serialization.XmlSerializer(typeof(HighlightedTabList));
                    tabList = serializer.Deserialize(stream) as HighlightedTabList;
                }
            }
            catch (System.Exception)
            {
            }

            model.SetTabs(tabList);
        }

        private string GetStoragePath()
        {
            string solutionSettingsDir = GetSolutionSettingsDir();
            if (solutionSettingsDir != null)
            {
                return System.IO.Path.Combine(solutionSettingsDir, GetStringHash(dte.Solution.FullName.ToUpperInvariant()) + ".marker");
            }
            return System.IO.Path.ChangeExtension(dte.Solution.FullName, "marker.tss");
        }

        private string GetSolutionSettingsDir()
        {
            try
            {
                return Microsoft.Win32.Registry.CurrentUser.OpenSubKey(@"Software\Sergey Vlasov\Tabs Studio").GetValue("SolutionSettingsDir") as string;
            }
           	catch(System.Exception)
            {
            }
            return null;
        }

        private string GetStringHash(string s)
        {
            System.Security.Cryptography.SHA1CryptoServiceProvider sha = new System.Security.Cryptography.SHA1CryptoServiceProvider();
            return System.BitConverter.ToString(sha.ComputeHash(System.Text.Encoding.Unicode.GetBytes(s))).Replace("-", "");
        }

        private Model model;
        private EnvDTE80.DTE2 dte;
    }
}
